@@ -23,10 +23,17 @@ have been moved here from the cclib repository when newer versions
 became available. We still want those logfiles to parse and test correctly,
 although sometimes special modification will be needed.
 
-To run the doctest, just use `python regression.py test`.
+To run the doctest, run `python -m test.regression` from the top level
+directory in the cclib repository.
 
-Note that this script was moved from the main cclib repository in Feb 2015
-in order for it to be close to the data, so look there for previous history.
+Running all regression can take anywhere from 10-20s to several minutes
+depending in your hardware. To aid debugging, there are two ways to limit
+which regressions to parse and test. You can limit the test to a specific
+parse, for example:
+    python -m test.regression Gaussian
+You can also limit a run to a single output file, using it's relative path
+inside the data directory, like so:
+    python -m test.regression Gaussian/Gaussian03/borane-opt.log
 """
 
 from __future__ import print_function
@@ -52,12 +59,14 @@ from cclib.parser import GAMESS
 from cclib.parser import GAMESSUK
 from cclib.parser import Gaussian
 from cclib.parser import Jaguar
+from cclib.parser import Molcas
 from cclib.parser import Molpro
 from cclib.parser import MOPAC
 from cclib.parser import NWChem
 from cclib.parser import ORCA
-from cclib.parser import Psi
+from cclib.parser import Psi4, Psi3
 from cclib.parser import QChem
+from cclib.parser import Turbomole
 
 from cclib.io import ccopen
 
@@ -268,6 +277,13 @@ def testGAMESS_GAMESS_US2013_N_UHF_out(logfile):
     """An UHF job that has an LZ value analysis between the alpha and beta orbitals."""
     assert len(logfile.data.moenergies) == 2
 
+def testGAMESS_GAMESS_US2014_CdtetraM1B3LYP_log(logfile):
+    """This logfile had coefficients for only 80 molecular orbitals."""
+    assert len(logfile.data.mocoeffs) == 2
+    assert numpy.count_nonzero(logfile.data.mocoeffs[0][79-1:, :]) == 258
+    assert numpy.count_nonzero(logfile.data.mocoeffs[0][80-1: 0:]) == 0
+    assert logfile.data.mocoeffs[0].all() == logfile.data.mocoeffs[1].all()
+
 def testGAMESS_WinGAMESS_dvb_td_trplet_2007_03_24_r1_out(logfile):
     """Do some basic checks for this old unit test that was failing.
 
@@ -510,18 +526,43 @@ def testGaussian_Gaussian09_benzene_freq_log(logfile):
     """Check that default precision vib displacements are parsed correctly."""
     assert abs(logfile.data.vibdisps[0,0,2] - (-0.04)) < 0.00001
 
+def testGaussian_Gaussian09_relaxed_PES_testH2_log(logfile):
+    """Check that all optimizations converge in a single step."""
+    atomcoords = logfile.data.atomcoords
+    optstatus = logfile.data.optstatus
+    assert len(optstatus) == len(atomcoords)
+
+    assert all(s == ccData.OPT_DONE + ccData.OPT_NEW for s in optstatus)
+
+def testGaussian_Gaussian09_relaxed_PES_testCO2_log(logfile):
+    """A relaxed PES scan with some uncoverged and some converged runs."""
+    atomcoords = logfile.data.atomcoords
+    optstatus = logfile.data.optstatus
+    assert len(optstatus) == len(atomcoords)
+
+    new_points = numpy.where(optstatus & ccData.OPT_NEW)[0]
+
+    # The first new point is just the beginning of the scan.
+    assert new_points[0] == 0
+
+    # The next two new points are at the end of unconverged runs.
+    assert optstatus[new_points[1]-1] == ccData.OPT_UNCONVERGED
+    assert all(optstatus[i] == ccData.OPT_UNKNOWN for i in range(new_points[0]+1, new_points[1]-1))
+    assert optstatus[new_points[2]-1] == ccData.OPT_UNCONVERGED
+    assert all(optstatus[i] == ccData.OPT_UNKNOWN for i in range(new_points[1]+1, new_points[2]-1))
+
+    # The next new point is after a convergence.
+    assert optstatus[new_points[3]-1] == ccData.OPT_DONE
+    assert all(optstatus[i] == ccData.OPT_UNKNOWN for i in range(new_points[2]+1, new_points[3]-1))
+    
+    # All subsequent point are both new and converged, since they seem
+    # to have converged in a single step.
+    assert all(s == ccData.OPT_DONE + ccData.OPT_NEW for s in optstatus[new_points[3]:])
+
 def testGaussian_Gaussian09_stopiter_gaussian_out(logfile):
     """Check to ensure that an incomplete SCF is handled correctly."""
     assert len(logfile.data.scfvalues[0]) == 4
 
-def testGaussian_Gaussian09_dvb_bomd_out(logfile):
-    """
-    Check that the number of energies and geometries corresponds to 
-    integrated steps, and not all the energies and geometries found in the file.
-    """
-    assert logfile.data.scfenergies.shape == (35,)
-    assert logfile.data.atomcoords.shape == (35,20,3)
-    assert logfile.data.time.shape == (35,)
 
 # Jaguar #
 
@@ -657,6 +698,13 @@ def testNWChem_NWChem6_5_stopiter_nwchem_hf_out(logfile):
     """Check to ensure that an incomplete SCF is handled correctly."""
     assert len(logfile.data.scfvalues[0]) == 2
 
+def testNWChem_NWChem6_8_526_out(logfile):
+    """If `print low` is present in the input, SCF iterations are not
+    printed.
+    """
+    assert not hasattr(logfile.data, "scftargets")
+    assert not hasattr(logfile.data, "scfvalues")
+
 # ORCA #
 
 def testORCA_ORCA2_8_co_cosmo_out(logfile):
@@ -727,7 +775,7 @@ def testORCA_ORCA4_0_IrCl6_sp_out(logfile):
 
 # PSI #
 
-def testPsi_Psi3_water_psi3_log(logfile):
+def testPsi3_Psi3_4_water_psi3_log(logfile):
     """An RHF for water with D orbitals and C2v symmetry.
 
     Here we can check that the D orbitals are considered by checking atombasis and nbasis.
@@ -735,21 +783,26 @@ def testPsi_Psi3_water_psi3_log(logfile):
     assert logfile.data.nbasis == 25
     assert [len(ab) for ab in logfile.data.atombasis] == [15, 5, 5]
 
-def testPsi_Psi4_0b5_dvb_gopt_hf_unconverged_out(logfile):
+def testPsi4_Psi4_beta5_dvb_gopt_hf_unconverged_out(logfile):
     """An unconverged geometry optimization to test for empty optdone (see #103 for details)."""
     assert hasattr(logfile.data, 'optdone') and not logfile.data.optdone
 
-def testPsi_Psi4_0b5_stopiter_psi_dft_out(logfile):
+def testPsi4_Psi4_beta5_stopiter_psi_dft_out(logfile):
     """Check to ensure that an incomplete SCF is handled correctly."""
     assert len(logfile.data.scfvalues[0]) == 7
 
-def testPsi_Psi4_0b5_stopiter_psi_hf_out(logfile):
+def testPsi4_Psi4_beta5_stopiter_psi_hf_out(logfile):
     """Check to ensure that an incomplete SCF is handled correctly."""
     assert len(logfile.data.scfvalues[0]) == 6
 
-def testPsi_Psi4_0_water_fdgrad_out(logfile):
+def testPsi4_Psi4_0_5_water_fdgrad_out(logfile):
     """Ensure that finite difference gradients are parsed."""
     assert hasattr(logfile.data, 'grads')
+    assert logfile.data.grads.shape == (1, 3, 3)
+    assert abs(logfile.data.grads[0, 0, 2] - 0.05498126903657) < 1.0e-12
+    # In C2v symmetry, there are 5 unique displacements for the
+    # nuclear gradient, and this is at the MP2 level.
+    assert logfile.data.mpenergies.shape == (5, 1)
 
 # Q-Chem #
 
@@ -1544,7 +1597,7 @@ class GaussianPolarTest(ReferencePolarTest):
     isotropic_delta = 2.0
     principal_components_delta = 0.7
 
-class JaguarSPTest_6_31gss(GenericSPTest):
+class JaguarSPTest_6_31gss(JaguarSPTest):
     """AO counts and some values are different in 6-31G** compared to STO-3G."""
     nbasisdict = {1: 5, 6: 15}
     b3lyp_energy = -10530
@@ -1571,7 +1624,7 @@ class JaguarGeoOptTest_6_31gss(GenericGeoOptTest):
 class MolproBigBasisTest_cart(MolproBigBasisTest):
     spherical = False
 
-class OrcaSPTest_3_21g(GenericSPTest_nosym):
+class OrcaSPTest_3_21g(OrcaSPTest, GenericSPTest_nosym):
     nbasisdict = {1: 2, 6: 9}
     b3lyp_energy = -10460
     overlap01 = 0.19
@@ -1693,7 +1746,7 @@ old_unittests = {
     "Gaussian/Gaussian09/dvb_scan_revA.02.log":         GaussianScanTest,
     "Gaussian/Gaussian09/dvb_sp_basis_b_gfprint.log":   GenericBasisTest,
     "Gaussian/Gaussian09/dvb_sp_basis_gfinput.log":     GenericBasisTest,
-    "Gaussian/Gaussian09/dvb_sp_revA.02.out":           GenericSPTest,
+    "Gaussian/Gaussian09/dvb_sp_revA.02.out":           GaussianSPTest,
     "Gaussian/Gaussian09/dvb_td_revA.02.out":           GaussianTDDFTTest,
     "Gaussian/Gaussian09/dvb_un_sp_revA.02.log":        GaussianSPunTest,
     "Gaussian/Gaussian09/dvb_un_sp_b_revA.02.log":      GaussianSPunTest,
@@ -1715,6 +1768,10 @@ old_unittests = {
     "Jaguar/Jaguar6.5/dvb_un_sp.out":   JaguarSPunTest,
     "Jaguar/Jaguar6.5/dvb_ir.out":      JaguarIRTest,
 
+    "Molcas/Molcas8.0/dvb_sp.out":      MolcasSPTest,
+    "Molcas/Molcas8.0/dvb_sp_un.out":   GenericSPunTest,
+    "Molcas/Molcas8.0/C_bigbasis.out":  MolcasBigBasisTest,
+
     "Molpro/Molpro2006/C_bigbasis_cart.out":    MolproBigBasisTest_cart,
     "Molpro/Molpro2012/trithiolane_polar.out":  GenericPolarTest,
 
@@ -1743,14 +1800,14 @@ old_unittests = {
 
     "ORCA/ORCA3.0/trithiolane_polar.out": GaussianPolarTest,
 
-    "Psi/Psi4.0b5/C_bigbasis.out":   GenericBigBasisTest,
-    "Psi/Psi4.0b5/dvb_gopt_hf.out":  PsiGeoOptTest,
-    "Psi/Psi4.0b5/dvb_sp_hf.out":    GenericBasisTest,
-    "Psi/Psi4.0b5/dvb_sp_hf.out":    GenericSPTest,
-    "Psi/Psi4.0b5/dvb_sp_ks.out":    GenericBasisTest,
-    "Psi/Psi4.0b5/dvb_sp_ks.out":    GenericSPTest,
-    "Psi/Psi4.0b5/water_ccsd.out":   GenericCCTest,
-    "Psi/Psi4.0b5/water_mp2.out":    GenericMP2Test,
+    "Psi4/Psi4-beta5/C_bigbasis.out":   GenericBigBasisTest,
+    "Psi4/Psi4-beta5/dvb_gopt_hf.out":  Psi4GeoOptTest,
+    "Psi4/Psi4-beta5/dvb_sp_hf.out":    GenericBasisTest,
+    "Psi4/Psi4-beta5/dvb_sp_hf.out":    PsiSPTest,
+    "Psi4/Psi4-beta5/dvb_sp_ks.out":    GenericBasisTest,
+    "Psi4/Psi4-beta5/dvb_sp_ks.out":    PsiSPTest,
+    "Psi4/Psi4-beta5/water_ccsd.out":   GenericCCTest,
+    "Psi4/Psi4-beta5/water_mp2.out":    GenericMP2Test,
 
     "QChem/QChem4.2/Trp_freq.out":           ReferencePolarTest,
     "QChem/QChem4.2/trithiolane_polar.out":  GaussianPolarTest,
@@ -1828,6 +1885,13 @@ def main(which=[], opt_traceback=False, opt_status=False, regdir=__regression_di
         orphaned = [t for t in tests if t[4:] not in normalized]
         orphaned_tests.extend(orphaned)
 
+    # Assume that if a string is not a parser name it'll be a relative
+    # path to a specific logfile.
+    # TODO: filter out things that are not parsers or files, and maybe
+    # raise an error in that case as well.
+    which_parsers = [w for w in which if w in parser_names]
+    which_filenames = [w for w in which if w not in which_parsers]
+
     failures = errors = total = 0
     for pn in parser_names:
 
@@ -1835,13 +1899,21 @@ def main(which=[], opt_traceback=False, opt_status=False, regdir=__regression_di
 
         # Continue to next iteration if we are limiting the regression and the current
         #   name was not explicitely chosen (that is, passed as an argument).
-        if len(which) > 0 and not pn in which:
+        if which_parsers and pn not in which_parsers:
             continue;
 
-        print("Are the %s files ccopened and parsed correctly?" % name)
+        parser_total = 0
         current_filenames = filenames[pn]
         current_filenames.sort()
         for fname in current_filenames:
+            relative_path = fname[len(regdir):]
+            if which_filenames and relative_path not in which_filenames:
+                continue;
+
+            parser_total += 1
+            if parser_total == 1:
+                print("Are the %s files ccopened and parsed correctly?" % pn)
+
             total += 1
             print("  %s ..."  % fname, end=" ")
 
@@ -1915,7 +1987,8 @@ def main(which=[], opt_traceback=False, opt_status=False, regdir=__regression_di
                 else:
                     print("test passed")
 
-        print()
+        if parser_total:
+            print()
 
     print("Total: %d   Failed: %d  Errors: %d" % (total, failures, errors))
     if not opt_traceback and failures + errors > 0:
